/***************************************************************************
 *   Copyright (C) 2004 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bpconf.h"
#include "applyjob.h"
#include "feedcheck.h"
#include "feeds.h"
#include "interval.h"

#include <qlayout.h>
#include <qlabel.h>

#include <klineedit.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <kurlcompletion.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <kmessagebox.h>
#include <kedittoolbar.h>
#include <klocale.h>
#include <kstatusbar.h>

BPConf::BPConf() : KMainWindow( 0, "MainWindow" )
{
   setupWindow();
   setupActions();
   setStandardToolBarMenuEnabled( true );
   createStandardStatusBarAction();
   setAutoSaveSettings();
   createGUI();

   loadOptions( instance()->config() );
   m_checkNow = false;
   m_logRead = false;
}

void BPConf::addFeed( const QString &url )
{
   m_pFeeds->addFeed( url );
}

bool BPConf::inLog( const QString &url )
{
   readLog();
   for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
   {
      if( url == *itr )
         return true;
   }
   return false;
}

void BPConf::addToLog( const QString &url )
{
   readLog();
   for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
   {
      if( url == *itr )
         return;
   }

   m_log += url;
}

void BPConf::remFromLog( const QString &url )
{
   readLog();
   for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
   {
      if( url == *itr )
         itr = m_log.erase( itr );
   }
}

void BPConf::saveProperties( KConfig *config )
{
   saveOptions( config );
}

void BPConf::readProperties( KConfig *config )
{
   loadOptions( config );
}

bool BPConf::queryClose()
{
   if( m_modified )
   {
      int ret = KMessageBox::warningYesNoCancel( 0, i18n( "The settings have been modified.\nDo you want to save your changes or discard them?" ),
                                       i18n( "Unsaved Changes" ), i18n( "&Save" ), i18n( "&Discard" ) );
      switch( ret )
      {
         case KMessageBox::Yes:
            slotApply();
            break;
         case KMessageBox::Cancel:
            return false;
            break;
      }
   }

   if( !m_pCheckNow->isEnabled() )
      KMessageBox::information( 0, i18n( "bashpodder is still gathering files from the feeds.\nIt will continue when you quit" ), i18n( "bashpodder" ) );

   return true;
}

void BPConf::fileNew()
{
   m_pDataDir->setText( "~/PodCasts/" );
   m_pFeeds->clear();
   m_pInterval->setAsDefault();
   setModified( false );
   m_logRead = false;
   m_log.clear();
}

void BPConf::importConfig()
{
   QString fileName = KFileDialog::getOpenFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
   if ( !fileName.isEmpty() )
   {
      m_pFeeds->loadFeeds( fileName );
      setModified( true );
   }
}

void BPConf::exportConfig()
{
   QString fileName = KFileDialog::getSaveFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
   if( !fileName.isEmpty() )
   {
      if ( QFile::exists( fileName ) )
      {
         if ( KMessageBox::warningContinueCancel( 0,
              i18n ("A document called \"%1\" already exists.\nDo you want to overwrite it?").arg( fileName ),
              QString::null, i18n ("Overwrite") ) == KMessageBox::Cancel )
            return;
      }

      m_pFeeds->saveFeeds( fileName );
   }
}

void BPConf::optionsConfigureToolbars()
{
   saveMainWindowSettings(KGlobal::config(), autoSaveGroup());
   KEditToolbar dlg(factory());
   connect(&dlg, SIGNAL(newToolbarConfig()), this, SLOT(applyNewToolbarConfig()));
   dlg.exec();
}

void BPConf::applyNewToolbarConfig()
{
   applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void BPConf::slotDataDirChanged()
{
   setModified( true );
   m_logRead = false;
   m_log.clear();
}

void BPConf::slotDataDir()
{
   KURL newDir = KDirSelectDialog::selectDirectory( m_pDataDir->text(), false, this, i18n( "Select Data Directory" ) );
   if( !newDir.isEmpty() )
   {
      m_pDataDir->setText( newDir.path( +1 ) );
      setModified( true );
      m_logRead = false;
      m_log.clear();
   }
}

void BPConf::slotCheckNow()
{
   if( m_modified )
   {
      if( KMessageBox::questionYesNo( 0, i18n( "Do you wish to apply the changes before checking?" ), i18n( "Apply Changes" ) ) == KMessageBox::Yes )
      {
         m_checkNow = true;
         slotApply();
         return;
      }
   }
   m_checkNow = false;
   m_pCheckNow->setEnabled( false );
   m_pFeedCheck = new FeedCheck( this, KURLCompletion::replacedPath( m_pDataDir->text(), true ) + "bashpodder.shell" );
   connect( m_pFeedCheck, SIGNAL( finished( bool ) ), SLOT( slotCheckNowFinish( bool ) ) );
   connect( m_pFeedCheck, SIGNAL( message( const QString& ) ), statusBar(), SLOT( message( const QString& ) ) );
   m_pFeedCheck->begin();
}

void BPConf::slotCheckNowFinish( bool success )
{
   if( success )
      statusBar()->message( i18n( "Finished checking podcasts" ), 2000 );
   else
      statusBar()->message( i18n( "There was an error while checking the feeds" ), 4000 );
   m_pCheckNow->setEnabled( true );
   delete m_pFeedCheck;
}

void BPConf::slotApply()
{
   m_pMain->setEnabled( false );
   QString path = KURLCompletion::replacedPath( m_pDataDir->text(), true );

   m_pApplyJob = new ApplyJob( this, m_pFeeds, path, m_pInterval->intervalString(), m_log.isEmpty() ? QString::null : m_log.join( "\n" ) );
   connect( m_pApplyJob, SIGNAL( success() ), SLOT( slotApplySuccess() ) );
   connect( m_pApplyJob, SIGNAL( failed( const QString& ) ), SLOT( slotApplyFailed( const QString& ) ) );
   connect( m_pApplyJob, SIGNAL( message( const QString& ) ), statusBar(), SLOT( message( const QString& ) ) );
   m_pApplyJob->begin();
}

void BPConf::slotApplySuccess()
{
   m_pMain->setEnabled( true );
   statusBar()->message( i18n( "Successfully applied changes" ), 2000 );
   setModified( false );
   saveOptions( instance()->config() );
   if( m_checkNow )
      slotCheckNow();
   delete m_pApplyJob;
}

void BPConf::slotApplyFailed( const QString &error )
{
   m_pMain->setEnabled( true );
   statusBar()->message( error, 4000 );
   delete m_pApplyJob;
}

void BPConf::slotReset()
{
   loadOptions( instance()->config() );
}

void BPConf::setupWindow()
{
   m_pMain = new QWidget( this );
   setCentralWidget( m_pMain );
   setCaption( i18n( "BashPodder" ) );

   m_pDirComplete = new KURLCompletion( KURLCompletion::DirCompletion );

   QVBoxLayout *top = new QVBoxLayout( m_pMain, KDialog::marginHint(), KDialog::spacingHint() );

   QHBoxLayout *hl = new QHBoxLayout( top );
   hl->addWidget( new QLabel( i18n( "Data Directory:" ), m_pMain ) );
   m_pDataDir = new KLineEdit( m_pMain );
   m_pDataDir->setURLDropsEnabled( true );
   m_pDataDir->setCompletionObject( m_pDirComplete, true );
   hl->addWidget( m_pDataDir );
   m_pDataDirButton = new KPushButton( SmallIcon( "fileopen" ), QString::null, m_pMain );
   hl->addWidget( m_pDataDirButton );

   m_pFeeds = new Feeds( m_pMain );
   top->addWidget( m_pFeeds );

   m_pInterval = new Interval( m_pMain );
   top->addWidget( m_pInterval );

   hl = new QHBoxLayout( top );
   m_pCheckNow = new KPushButton( i18n( "Check Now" ), m_pMain );
   hl->addWidget( m_pCheckNow );
   hl->addStretch();
   m_pApply = new KPushButton( i18n( "Apply" ), m_pMain );
   m_pApply->setEnabled( false );
   hl->addWidget( m_pApply );
   m_pReset = new KPushButton( i18n( "Reset" ), m_pMain );
   m_pReset->setEnabled( false );
   hl->addWidget( m_pReset );

   connect( m_pDataDir, SIGNAL( textChanged( const QString& ) ), SLOT( slotDataDirChanged() ) );
   connect( m_pDataDirButton, SIGNAL( clicked() ), SLOT( slotDataDir() ) );
   connect( m_pFeeds, SIGNAL( dataChanged() ), SLOT( slotDataChanged() ) );
   connect( m_pInterval, SIGNAL( dataChanged() ), SLOT( slotDataChanged() ) );
   connect( m_pCheckNow, SIGNAL( clicked() ), SLOT( slotCheckNow() ) );
   connect( m_pApply, SIGNAL( clicked() ), SLOT( slotApply() ) );
   connect( m_pReset, SIGNAL( clicked() ), SLOT( slotReset() ) );
}

void BPConf::setupActions()
{
   KStdAction::openNew( this, SLOT(fileNew()), actionCollection() );
   KStdAction::quit( this, SLOT(close()), actionCollection() );
   new KAction( i18n( "Import configuration file..." ), "fileopen", 0, this, SLOT(importConfig()), actionCollection(), "import_config" );
   new KAction( i18n( "Export configuration file..." ), "filesaveas", 0, this, SLOT(exportConfig()), actionCollection(), "export_config" );

   // settings menu
   KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
}

void BPConf::setModified( bool modified )
{
   m_modified = modified;
   m_pApply->setEnabled( modified );
   m_pReset->setEnabled( modified );
}

void BPConf::loadOptions( KConfig *config )
{
   config->setGroup( "options" );
   m_pDataDir->setText( config->readEntry( "data_dir", "~/PodCasts/" ) );
   m_pInterval->loadInterval( config );
   m_pFeeds->loadFeeds( config );
   m_modified = config->readBoolEntry( "modified", false );
   setModified( m_modified );
}

void BPConf::saveOptions( KConfig *config )
{
   config->deleteGroup( "options" );
   config->setGroup( "options" );
   config->writeEntry( "data_dir", m_pDataDir->text() );
   m_pInterval->saveInterval( config );
   m_pFeeds->saveFeeds( config );
   config->writeEntry( "modified", m_modified );
}

void BPConf::readLog()
{
   if( m_logRead )
      return;

   QString fileName = KURLCompletion::replacedPath( m_pDataDir->text(), true ) + "podcast.log";
   QFile file( fileName );
   if( file.open( IO_ReadOnly ) )
   {
      QTextStream stream( &file );
      while( !stream.atEnd() )
         m_log += stream.readLine().simplifyWhiteSpace();

      file.close();
   }

   m_logRead = true;
}

#include "bpconf.moc"
