/***************************************************************************
 *   Copyright (C) 2004 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "feeds.h"
#include "feeddialog.h"
#include "feeditem.h"

#include <qlayout.h>
#include <qfile.h>

#include <klistview.h>
#include <kpushbutton.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdialog.h>
#include <kconfig.h>

Feeds::Feeds(QWidget *parent, const char *name) : QWidget( parent, name )
{
   QHBoxLayout *hl = new QHBoxLayout( this, 0, KDialog::spacingHint() );
   m_pFeeds = new KListView( this );
   m_pFeeds->addColumn( i18n( "Name" ) );
   m_pFeeds->addColumn( i18n( "URL" ) );
   hl->addWidget( m_pFeeds );
   QVBoxLayout *vl = new QVBoxLayout( hl );
   m_pFeedAdd = new KPushButton( i18n( "Add" ), this );
   m_pFeedMod = new KPushButton( i18n( "Modify" ), this );
   m_pFeedRem = new KPushButton( i18n( "Delete" ), this );
   vl->addWidget( m_pFeedAdd );
   vl->addWidget( m_pFeedMod );
   vl->addWidget( m_pFeedRem );
   vl->addStretch();

   connect( m_pFeeds, SIGNAL( selectionChanged() ), SLOT( slotFeedSelect() ) );
   connect( m_pFeeds, SIGNAL( executed( QListViewItem* ) ), SLOT( slotFeedExec( QListViewItem* ) ) );
   connect( m_pFeedAdd, SIGNAL( clicked() ), SLOT( slotAddFeed() ) );
   connect( m_pFeedMod, SIGNAL( clicked() ), SLOT( slotModFeed() ) );
   connect( m_pFeedRem, SIGNAL( clicked() ), SLOT( slotRemFeed() ) );

   slotFeedSelect();
}

void Feeds::addFeed( const QString &url )
{
   FeedItem *item = new FeedItem( m_pFeeds );
   item->setURL( url );
   FeedDialog *fd = new FeedDialog( this, item );
   int code = fd->exec();
   if( code == QDialog::Accepted )
      emit dataChanged();
   else
      delete item;
}

bool Feeds::loadFeeds( const QString &fileName )
{
   bool retNames = false;
   switch( KMessageBox::questionYesNoCancel( 0, i18n( "Should feeed names be retrieved from the net if possible?" ), i18n( "Retrieve Names" ) ) )
   {
      case KMessageBox::Yes:
         retNames = true;
         break;
      case KMessageBox::Cancel:
         return false;
         break;
   }

   FeedItem *item;
   QFile file( fileName );
   if( file.open( IO_ReadOnly ) )
   {
      QString url, name;
      QTextStream stream( &file );
      int num = 0;
      while( !stream.atEnd() )
      {
         url = stream.readLine().simplifyWhiteSpace();
         if( num < 10 )
            name = QString( "Imported 0%1" ).arg( num++ );
         else
            name = QString( "Imported %1" ).arg( num++ );

         item = new FeedItem( m_pFeeds, name, url );
         if( retNames )
            item->getNameFromURL();
      }
      file.close();
      return true;
   }
   else
   {
      KMessageBox::error( 0, i18n( "Couldn't open the file %1" ).arg( fileName ), i18n( "Couldn't Open" ) );
      return false;
   }
}

void Feeds::loadFeeds( KConfig *config )
{
   int numFeeds = config->readNumEntry( "number_of_feeds" );
   QString feedtag;
   m_pFeeds->clear();
   for( int i = 0; i < numFeeds; ++i )
   {
      feedtag = "feed" + QString::number( i );
      new FeedItem( m_pFeeds, config->readEntry( feedtag + "_name" ), config->readEntry( feedtag + "_url" ) );
   }
}

bool Feeds::saveFeeds( const QString &fileName )
{
   QFile file( fileName );
   if( file.open( IO_WriteOnly ) )
   {
      QTextStream stream( &file );

      int numFeeds = m_pFeeds->childCount();
      FeedItem *item;
      for( int i = 0; i < numFeeds; ++i )
      {
         item = dynamic_cast<FeedItem*>( m_pFeeds->itemAtIndex( i ) );
         stream << item->url() << "\n";
      }
      file.close();
      return true;
   }
   else
   {
      KMessageBox::error( 0, i18n( "Couldn't save the file %1" ).arg( fileName ), i18n( "Couldn't save" ) );
      return false;
   }
}

void Feeds::saveFeeds( KConfig *config )
{
   int numFeeds = m_pFeeds->childCount();
   config->writeEntry( "number_of_feeds", numFeeds );
   QString feedtag;
   FeedItem *item;
   for( int i = 0; i < numFeeds; ++i )
   {
      feedtag = "feed" + QString::number( i );
      item = dynamic_cast<FeedItem*>( m_pFeeds->itemAtIndex( i ) );
      config->writeEntry( feedtag + "_name", item->name() );
      config->writeEntry( feedtag + "_url", item->url() );
   }
}

void Feeds::clear()
{
   m_pFeeds->clear();
   slotFeedSelect();
}

void Feeds::slotFeedSelect()
{
   FeedItem *item = dynamic_cast<FeedItem*>( m_pFeeds->selectedItem() );
   if( item )
   {
      m_pFeedMod->setEnabled( true );
      m_pFeedRem->setEnabled( true );
   }
   else
   {
      m_pFeedMod->setEnabled( false );
      m_pFeedRem->setEnabled( false );
   }
}

void Feeds::slotFeedExec( QListViewItem *item )
{
   FeedItem *Fitem = dynamic_cast<FeedItem*>( item );
   FeedDialog *fd = new FeedDialog( this, Fitem );
   int code = fd->exec();
   if( code == QDialog::Accepted )
      emit dataChanged();
}

void Feeds::slotAddFeed()
{
   FeedItem *item = new FeedItem( m_pFeeds );
   FeedDialog *fd = new FeedDialog( this, item );
   int code = fd->exec();
   if( code == QDialog::Accepted )
      emit dataChanged();
   else
      delete item;
}

void Feeds::slotModFeed()
{
   FeedItem *item = dynamic_cast<FeedItem*>( m_pFeeds->selectedItem() );
   if( !item )
   {
      slotFeedSelect();
      return;
   }

   FeedDialog *fd = new FeedDialog( this, item );
   int code = fd->exec();
   if( code == QDialog::Accepted )
      emit dataChanged();
}

void Feeds::slotRemFeed()
{
   FeedItem *item = dynamic_cast<FeedItem*>( m_pFeeds->selectedItem() );
   if( !item )
   {
      slotFeedSelect();
      return;
   }

   delete item;
   emit dataChanged();
   slotFeedSelect();
}

#include "feeds.moc"
