/***************************************************************************
 *   Copyright (C) 2004 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "interval.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>

#include <klineedit.h>
#include <kpushbutton.h>
#include <ktimewidget.h>
#include <kconfig.h>
#include <klocale.h>
#include <kdialog.h>

Interval::Interval(QWidget *parent, const char *name) : QWidget(parent, name)
{
   QHBoxLayout *hl = new QHBoxLayout( this, 0, KDialog::spacingHint() );
   hl->addWidget( new QLabel( i18n( "Check every:" ), this ) );
   m_pIntervalType = new QComboBox( this );
   m_pIntervalType->insertItem( i18n( "Hour" ) );
   m_pIntervalType->insertItem( i18n( "Day" ) );
   m_pIntervalType->insertItem( i18n( "Week" ) );
   m_pIntervalType->insertItem( i18n( "Month" ) );
   m_pIntervalType->insertItem( i18n( "Never" ) );
   hl->addWidget( m_pIntervalType );
   m_pOnLabel = new QLabel( i18n( "on" ), this );
   m_pOnLabel->hide();
   hl->addWidget( m_pOnLabel );
   m_pDay = new QComboBox( this );
   m_pDay->insertItem( i18n( "Sunday" ) );
   m_pDay->insertItem( i18n( "Monday" ) );
   m_pDay->insertItem( i18n( "Tuesday" ) );
   m_pDay->insertItem( i18n( "Wednesday" ) );
   m_pDay->insertItem( i18n( "Thursday" ) );
   m_pDay->insertItem( i18n( "Friday" ) );
   m_pDay->insertItem( i18n( "Saturday" ) );
   m_pDay->hide();
   hl->addWidget( m_pDay );
   m_pDates = new KLineEdit( this );
   m_pDates->setValidator( new DateLineValid( m_pDates ) );
   m_pDates->hide();
   hl->addWidget( m_pDates );
   m_pAtLabel = new QLabel( i18n( "at" ), this );
   hl->addWidget( m_pAtLabel );
   m_pTime = new KTimeWidget( QTime( 0, 0 ) ,this );
   hl->addWidget( m_pTime );
   hl->addStretch();

   connect( m_pIntervalType, SIGNAL( activated( int ) ), SLOT( slotIntervalType( int ) ) );
   connect( m_pDay, SIGNAL( activated( int ) ), SIGNAL( dataChanged() ) );
   connect( m_pDates, SIGNAL( textChanged( const QString& ) ), SIGNAL( dataChanged() ) );
   connect( m_pDates, SIGNAL( lostFocus() ), SLOT( slotDatesValid() ) );
   connect( m_pTime, SIGNAL( valueChanged( const QTime& ) ), SIGNAL( dataChanged() ) );
}

void Interval::loadInterval( KConfig *config )
{
   m_pIntervalType->setCurrentItem( config->readNumEntry( "check_interval", 1 ) );
   slotIntervalType( m_pIntervalType->currentItem() );
   m_pDay->setCurrentItem( config->readNumEntry( "check_on_day" ) );
   m_pDates->setText( config->readEntry( "check_on_date" ) );
   m_pTime->setTime( QTime::fromString( config->readEntry( "check_time", "00:00" ) ) );
}

void Interval::saveInterval( KConfig *config )
{
   config->writeEntry( "check_interval", m_pIntervalType->currentItem() );
   config->writeEntry( "check_on_day", m_pDay->currentItem() );
   config->writeEntry( "check_on_date", m_pDates->text() );
   config->writeEntry( "check_time", m_pTime->time().toString() );
}

void Interval::setAsDefault()
{
   m_pIntervalType->setCurrentItem( 1 );
   slotIntervalType( 1 );
   m_pTime->setTime( QTime( 0, 0 ) );
}

QString Interval::intervalString()
{
   QString str;
   QTime time = m_pTime->time();
   switch( m_pIntervalType->currentItem() )
   {
      case 0: // every hour
         str = "0 * * * * ";
         break;
      case 1: // every day
         str = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + " * * * ";
         break;
      case 2: // every week
         str = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + " * * " +
               QString::number( m_pDay->currentItem() ) + " ";
         break;
      case 3: // every month
         str = QString::number( time.minute() ) + " " + QString::number( time.hour() ) + " " +
               m_pDates->text() + " * * ";
         break;
      case 4: // Never
         str = QString::null;
         break;
   }
   return str;
}

void Interval::slotIntervalType( int item )
{
   switch( item )
   {
      case 0: // Every Hour
      case 4: // Never
         m_pOnLabel->hide();
         m_pDay->hide();
         m_pDates->hide();
         m_pAtLabel->hide();
         m_pTime->hide();
         break;
      case 1: // Every Day
         m_pOnLabel->hide();
         m_pDay->hide();
         m_pDates->hide();
         m_pAtLabel->show();
         m_pTime->show();
         break;
      case 2: // Every Week
         m_pOnLabel->show();
         m_pDay->show();
         m_pDates->hide();
         m_pAtLabel->show();
         m_pTime->show();
         break;
      case 3: // Every Month
         m_pOnLabel->show();
         m_pDay->hide();
         m_pDates->show();
         m_pAtLabel->show();
         m_pTime->show();
         break;
   }
   emit dataChanged();
}

void Interval::slotDatesValid()
{
   if( !m_pDates->hasAcceptableInput() )
   {
      QString text = m_pDates->text();
      m_pDates->validator()->fixup( text );
      m_pDates->setText( text );
      if( !m_pDates->hasAcceptableInput() )
         m_pDates->setFocus();
   }
}

DateLineValid::State DateLineValid::validate( QString &input, int& ) const
{
   if( input.isEmpty() )
      return Intermediate;

   QRegExp rxp = QRegExp( "^[\\d,]+$" );
   if( !input.contains( rxp ) || input.startsWith( "," ) || input.contains( ",," ) )
      return Invalid;

   QStringList values = QStringList::split( ",", input );

   bool ok;
   int num;
   for( QStringList::Iterator itr = values.begin(); itr != values.end(); ++itr )
   {
      num = (*itr).toInt( &ok );
      if( !ok || num < 1 || num > 31 )
         return Invalid;
   }

   if( input.endsWith( "," ) )
      return Intermediate;
   else
      return Acceptable;
}

void DateLineValid::fixup( QString &input ) const
{
   input = input.simplifyWhiteSpace();
   input.remove( " " );

   if( input.endsWith( "," ) )
      input.truncate( input.length() -1 );
}

#include "interval.moc"
