/***************************************************************************
 *   Copyright (C) 2004 by Grzegorz Rusin                                  *
 *   pks@irc.pl                                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "header.h"

QChart::QChart(QWidget *parent, const char *name, WFlags f) : QWidget(parent, name, f)
{
	loadSettings();
	points = NULL;

	QPainter paint;
	QPixmap *px;

	//hide button
	hideButton = new QSimpleButton(this);
	hideButton->resize(13, 13);
	px = new QPixmap(hideButton->size());
	hideButton->setPixmap(*px);
	redrawHideButton();
	hideButton->show();

	connect(hideButton, SIGNAL(clicked()), SLOT(hide()));

	//minimize button
	minimizeButton = new QSimpleButton(this);
	minimizeButton->resize(13, 13);
	px = new QPixmap(minimizeButton->size());
	minimizeButton->setPixmap(*px);
	redrawMinimizeButton();
	minimizeButton->show();

	connect(minimizeButton, SIGNAL(clicked()), SLOT(showMinimized()));

	drag = 0;

	QToolTip::add(minimizeButton, "Minimizie");
	QToolTip::add(hideButton, "Hide graph");
}

QChart::~QChart()
{
	delete hideButton;
	delete minimizeButton;
	if(points) delete points;
	saveSettings();
}

void QChart::paintEvent(QPaintEvent *)
{
	bitBlt(this, 0, 0, &bb, 0, 0, width(), height(), Qt::CopyROP);
}

void QChart::resizeEvent(QResizeEvent *)
{
	bb.resize(width(), height());
	bb.fill(backgroundColor);

	hideButton->move(width()-20, 5);
	minimizeButton->move(hideButton->x()-minimizeButton->width()-5, 5);
	update();
}

void QChart::drawGrid(QPainter *p)
{
	QPainter *paint;
	QString str;
	if(p) paint = p;
	else paint = new QPainter(&bb);

	for(int i=0; i<height(); i+=PX_PER_DEG*10)
	{
		//draw grid
		paint->setPen(gridColor);
		paint->drawLine(0, height()-i, width()-1, height()-i);

		//draw labels
		paint->setPen(textColor);
		paint->setFont(textFont);
		paint->drawText(0, height()-i-PX_PER_DEG-textFont.pointSize(), width()-1, height()-i, AlignRight, QString("%1 C").arg(i/PX_PER_DEG));
	}

	if(!p) delete paint;
}

void QChart::update(cArray<int> *array)
{
	int i;

	if(array)
	{
		if(points) delete points;

		if(array->entries() > 1) points = new QPointArray(array->entries()+2);
		else points = new QPointArray(array->entries()+1);

		points->setPoint(0, width(), height()-(*array)[0]*PX_PER_DEG);

		for(i=1; i<array->entries()-1; ++i)
			points->setPoint(i, width()-1-i*PX_PER_PROBE, height()-(*array)[i]*PX_PER_DEG);

		int n;
		if(array->entries() > 1)
		{
			n = width()-1-i*PX_PER_PROBE;
			if(n < PX_PER_PROBE) n = -1;

			points->setPoint(i++, n, height()-(*array)[i]*PX_PER_DEG);
			points->setPoint(i++, n, height());
		}
		points->setPoint(i, width(), height());
	}

	emit newData();

	if(bb.isNull()) return;

	QPainter paint(&bb);
	QPen pen;

	//clear
	bb.fill(backgroundColor);

	//draw grid
	drawGrid();

	//draw points
	if(points)
	{
		QBrush brush(graphColor, Qt::Dense4Pattern);
		paint.setBrush(brush);

		pen.setWidth(1);
		pen.setColor(graphColor);
		paint.setPen(pen);
		paint.drawPolygon(*points, TRUE);
	}

	QString comment;

	//draw comment
	paint.setPen(textColor);
	paint.setFont(textFont);

	if(points) paint.drawText(PX_PER_PROBE*3, PX_PER_DEG*5,
			QString("last probe: %1 C").arg(lastProbe()));
	paint.end();

	repaint(FALSE);

	redrawHideButton();
	redrawMinimizeButton();
}

void QChart::setBackgroundColor()
{
	backgroundColor = QColorDialog::getRgba(backgroundColor.rgb());
	update();
}

void QChart::setGraphColor()
{
	graphColor = QColorDialog::getRgba(graphColor.rgb());
	update();
}

void QChart::setGridColor()
{
	gridColor = QColorDialog::getRgba(gridColor.rgb());
	update();
}

void QChart::setTextColor()
{
	textColor = QColorDialog::getRgba(textColor.rgb());
	update();
}

void QChart::setTextFont()
{
	bool ok;
	QFont tmp = textFont;

	tmp = QFontDialog::getFont(&ok, tmp, this );

	if(ok)
	{
		textFont = tmp;
		update();
	}
}

void QChart::mousePressEvent(QMouseEvent *e)
{
	if(e->button() == Qt::RightButton)
	{
		QPopupMenu* contextMenu = new QPopupMenu( this );
		contextMenu->insertItem("change &background color",  this, SLOT(setBackgroundColor()), CTRL+Key_B);
		contextMenu->insertItem("change g&raph color",  this, SLOT(setGraphColor()), CTRL+Key_R);
		contextMenu->insertItem("change &grid color",  this, SLOT(setGridColor()), CTRL+Key_G);
		contextMenu->insertItem("change &text color",  this, SLOT(setTextColor()), CTRL+Key_T);
		contextMenu->insertItem("change text &font",  this, SLOT(setTextFont()), CTRL+Key_F);
		contextMenu->insertItem("&close graph", this, SLOT(hide()), CTRL+Key_C);
		contextMenu->exec(QCursor::pos());
		delete contextMenu;
		update();
	}
	else if(e->button() == Qt::LeftButton)
	{
		lastMousePos = QCursor::pos();
		drag = TRUE;
	}
}

void QChart::mouseReleaseEvent(QMouseEvent *e)
{
	if(e->button() == Qt::LeftButton)
	{
		drag = FALSE;
	}
}

void QChart::mouseMoveEvent(QMouseEvent *e)
{
	if(drag)
	{
		QPoint p = mapToGlobal(pos());

		move(p.x() + (e->x()-lastMousePos.x()),
			 p.y() + (e->y()-lastMousePos.y()));

		lastMousePos = QCursor::pos();
	}

}

void QChart::redrawHideButton()
{
	QPixmap *px = hideButton->pixmap();
	QPainter paint(px);;

	px->fill(backgroundColor);
	paint.setPen(QPen(gridColor, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
	paint.drawLine(2, 2,  px->width()-2, px->height()-2);
	paint.drawLine(2, px->height()-2, px->width()-2,  2);
	hideButton->setPixmap(*px);
}

void QChart::redrawMinimizeButton()
{
	QPixmap *px = minimizeButton->pixmap();
	QPainter paint(px);;

	px->fill(backgroundColor);
	paint.setPen(QPen(gridColor, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
	paint.drawLine(2, px->height()-2,  px->width()-2, px->height()-2);
	minimizeButton->setPixmap(*px);
}

void QChart::saveSettings()
{
	QSettings settings;

	settings.insertSearchPath(QSettings::Windows, WINDOWS_REGISTRY);
	settings.writeEntry(APP_KEY + "backgroundColor", backgroundColor.name());
	settings.writeEntry(APP_KEY + "gridColor", gridColor.name());
	settings.writeEntry(APP_KEY + "textColor", textColor.name());
	settings.writeEntry(APP_KEY + "graphColor", graphColor.name());
	settings.writeEntry(APP_KEY + "textFont", textFont.toString());

	QPoint p = pos();

	settings.writeEntry(APP_KEY + "windowPosX", QString::number(p.x()));
	settings.writeEntry(APP_KEY + "windowPosY", QString::number(p.y()));
}

void QChart::loadSettings()
{
	QSettings settings;

	settings.insertSearchPath(QSettings::Windows, WINDOWS_REGISTRY);

	backgroundColor = QColor(settings.readEntry(APP_KEY + "backgroundColor", "#013e19"));
	gridColor = QColor(settings.readEntry(APP_KEY + "gridColor", "#129a00"));
	textColor = QColor(settings.readEntry(APP_KEY + "textColor", "#0deb1c"));
	graphColor = QColor(settings.readEntry(APP_KEY + "graphColor", "#ffff00"));
	textFont.fromString(settings.readEntry(APP_KEY + "textFont", "Helvetica,12,-1,5,75,0,0,0,0,0"));

	QPoint p;
	p.setX(settings.readNumEntry(APP_KEY + "windowPosX"));
	p.setY(settings.readNumEntry(APP_KEY + "windowPosY"));
	move(p);
}

int QChart::lastProbe()
{
	if(points) return (height()-points->point(0).y())/PX_PER_DEG;
	else return 0;
}
